"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCharacterPortraitPNGFilePath = exports.getCharacterNamePNGFilePath = exports.getBossPortraitPNGFilePath = exports.getBossNamePNGFilePath = void 0;
const bossNamePNGFileNames_1 = require("../objects/bossNamePNGFileNames");
const bossPortraitPNGFileNames_1 = require("../objects/bossPortraitPNGFileNames");
const playerNamePNGFileNames_1 = require("../objects/playerNamePNGFileNames");
const playerPortraitPNGFileNames_1 = require("../objects/playerPortraitPNGFileNames");
/** Most of the PNG files related to the versus screen are located in this directory. */
const PNG_PATH_PREFIX = "gfx/ui/boss";
/**
 * Player portraits are not located in the same directory as everything else, since they are re-used
 * from the animation where the player travels to a new stage.
 */
const PLAYER_PORTRAIT_PNG_PATH_PREFIX = "gfx/ui/stage";
/**
 * Helper function to get the path to the name file that corresponds to the graphic shown on the
 * versus screen for the particular boss.
 *
 * For example, the file path for `BossID.MONSTRO` is "gfx/ui/boss/bossname_20.0_monstro.png".
 */
function getBossNamePNGFilePath(bossID) {
    const fileName = bossNamePNGFileNames_1.BOSS_NAME_PNG_FILE_NAMES[bossID];
    return `${PNG_PATH_PREFIX}/${fileName}`;
}
exports.getBossNamePNGFilePath = getBossNamePNGFilePath;
/**
 * Helper function to get the path to the portrait file that corresponds to the graphic shown on the
 * versus screen for the particular boss.
 *
 * For example, the file path for `BossID.MONSTRO` is "gfx/ui/boss/portrait_20.0_monstro.png".
 */
function getBossPortraitPNGFilePath(bossID) {
    const fileName = bossPortraitPNGFileNames_1.BOSS_PORTRAIT_PNG_FILE_NAMES[bossID];
    return `${PNG_PATH_PREFIX}/${fileName}`;
}
exports.getBossPortraitPNGFilePath = getBossPortraitPNGFilePath;
/**
 * Helper function to get the path to the name file that corresponds to the graphic shown on the
 * versus screen for the particular character.
 *
 * For example, the file path for `PlayerType.ISAAC` is "gfx/ui/boss/playername_01_isaac.png".
 */
function getCharacterNamePNGFilePath(character) {
    const fileName = playerNamePNGFileNames_1.PLAYER_NAME_PNG_FILE_NAMES[character];
    return `${PNG_PATH_PREFIX}/${fileName}`;
}
exports.getCharacterNamePNGFilePath = getCharacterNamePNGFilePath;
/**
 * Helper function to get the path to the portrait file that corresponds to the graphic shown on the
 * versus screen for the particular character.
 *
 * For example, the file path for `PlayerType.ISAAC` is "gfx/ui/boss/playerportrait_isaac.png".
 */
function getCharacterPortraitPNGFilePath(character) {
    const fileName = playerPortraitPNGFileNames_1.PLAYER_PORTRAIT_PNG_FILE_NAMES[character];
    return `${PLAYER_PORTRAIT_PNG_PATH_PREFIX}/${fileName}`;
}
exports.getCharacterPortraitPNGFilePath = getCharacterPortraitPNGFilePath;
