"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ROOM_SHAPE_TO_DOOR_SLOT_COORDINATES = void 0;
const isaac_typescript_definitions_1 = require("isaac-typescript-definitions");
/**
 * The coordinates correspond to the x and y values that are present in a room's XML file.
 *
 * e.g. `<door exists="False" x="-1" y="3" />`
 */
exports.ROOM_SHAPE_TO_DOOR_SLOT_COORDINATES = {
    // 1
    [isaac_typescript_definitions_1.RoomShape.SHAPE_1x1]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [13, 3],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 7], // 3
    },
    // 2
    [isaac_typescript_definitions_1.RoomShape.IH]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [13, 3], // 2
    },
    // 3
    [isaac_typescript_definitions_1.RoomShape.IV]: {
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 7], // 3
    },
    // 4
    [isaac_typescript_definitions_1.RoomShape.SHAPE_1x2]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [13, 3],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 14],
        [isaac_typescript_definitions_1.DoorSlot.LEFT_1]: [-1, 10],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_1]: [13, 10], // 6
    },
    // 5
    [isaac_typescript_definitions_1.RoomShape.IIV]: {
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 14], // 3
    },
    // 6
    [isaac_typescript_definitions_1.RoomShape.SHAPE_2x1]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [26, 3],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 7],
        [isaac_typescript_definitions_1.DoorSlot.UP_1]: [19, -1],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_1]: [19, 7], // 7
    },
    // 7
    [isaac_typescript_definitions_1.RoomShape.IIH]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [26, 3], // 2
    },
    // 8
    [isaac_typescript_definitions_1.RoomShape.SHAPE_2x2]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [26, 3],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 14],
        [isaac_typescript_definitions_1.DoorSlot.LEFT_1]: [-1, 10],
        [isaac_typescript_definitions_1.DoorSlot.UP_1]: [19, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_1]: [26, 10],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_1]: [19, 14], // 7
    },
    // 9
    [isaac_typescript_definitions_1.RoomShape.LTL]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [12, 3],
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, 6],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [26, 3],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 14],
        [isaac_typescript_definitions_1.DoorSlot.LEFT_1]: [-1, 10],
        [isaac_typescript_definitions_1.DoorSlot.UP_1]: [19, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_1]: [26, 10],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_1]: [19, 14], // 7
    },
    // 10
    [isaac_typescript_definitions_1.RoomShape.LTR]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [13, 3],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 14],
        [isaac_typescript_definitions_1.DoorSlot.LEFT_1]: [-1, 10],
        [isaac_typescript_definitions_1.DoorSlot.UP_1]: [19, 6],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_1]: [26, 10],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_1]: [19, 14], // 7
    },
    // 11
    [isaac_typescript_definitions_1.RoomShape.LBL]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [26, 3],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 7],
        [isaac_typescript_definitions_1.DoorSlot.LEFT_1]: [12, 10],
        [isaac_typescript_definitions_1.DoorSlot.UP_1]: [19, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_1]: [26, 10],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_1]: [19, 14], // 7
    },
    // 12
    [isaac_typescript_definitions_1.RoomShape.LBR]: {
        [isaac_typescript_definitions_1.DoorSlot.LEFT_0]: [-1, 3],
        [isaac_typescript_definitions_1.DoorSlot.UP_0]: [6, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_0]: [26, 3],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_0]: [6, 14],
        [isaac_typescript_definitions_1.DoorSlot.LEFT_1]: [-1, 10],
        [isaac_typescript_definitions_1.DoorSlot.UP_1]: [19, -1],
        [isaac_typescript_definitions_1.DoorSlot.RIGHT_1]: [13, 10],
        [isaac_typescript_definitions_1.DoorSlot.DOWN_1]: [19, 7], // 7
    },
};
